/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#ifndef IMPORTNATIONALFLIGHTDATABASE_H_
#define IMPORTNATIONALFLIGHTDATABASE_H_

#include <iostream>
#include <fstream>
#include <sys/time.h>
#include <gtkmm/window.h>
#include <gtkmm/progressbar.h>
#include <gtkmm/button.h>
#include <libglademm.h>
#include "Config.h"
#include "NFD.h"
#include "DataStore.h"

class ImportNationalFlightDatabase
{
public:
	ImportNationalFlightDatabase(Config *Cfg, std::string *GladeFilename, DataStore *Data);
	~ImportNationalFlightDatabase();

	int Run();
	bool ReadLine();
	void on_CancelButton_clicked();
	void ScheduleMainLoop();
	bool ConfirmData();

private:
	static const double ReadingProgressFrac=0.5;
	static const double ParseAirportsProgressFrac=0.1;
	static const double ParseRunwaysProgressFrac=0.1;
	static const double ParseVORsProgressFrac=0.1;
	static const double ParseNDBsProgressFrac=0.1;
	static const double ParseWaypointsProgressFrac=0.1;

	Glib::RefPtr<Gnome::Glade::Xml> refXml;
	Config *Cfg;
	std::string *GladeFilename;
	Gtk::Window *ProgressWindow;
	Gtk::ProgressBar *ProgressBar,*SubProgressBar;
	Gtk::Button *CancelButton;
	int BytesRead,BytesAvail;
	std::ifstream IFStream;
	struct timeval LastProgressBarUpdate;
	bool CancelFlag;
	NFD NFDData;
	DataStore *Data;
	Volume *VolData;
	double ProgressDone;
};

#endif /*IMPORTNATIONALFLIGHTDATABASE_H_*/
