/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#ifdef HAVE_CONFIG_H
 #include "../config.h"
#endif

#ifndef MAINWINDOW_H_
#define MAINWINDOW_H_

#include <time.h>
#include <string>
#include <list>
#include <gtkmm/window.h>
#include <gtkmm/menuitem.h>
#include <gtkmm/menu.h>
#include <gtkmm/statusbar.h>
#include <libglademm.h>

#ifdef HAVE_LIBDBUS_1
#include <dbus/dbus.h>
#ifdef HAVE_LIBDBUS_GLIB_1
#include <dbus/dbus-glib.h>
#endif
#endif

#include "vifir.h"
#include "ChartDrawingArea.h"
#include "GPS.h"
#include "Config.h"
#include "RouteDialog.h"
#include "Route.h"
#include "DataStore.h"
#include "DataManagerDialog.h"
#include "Aircraft.h"
#include "METARs.h"
#include "Shapefile.h"

#define MAINWINDOW_DBUS_SS_SERVICE   "org.gnome.ScreenSaver"
#define MAINWINDOW_DBUS_SS_PATH      "/org/gnome/ScreenSaver"
#define MAINWINDOW_DBUS_SS_INTERFACE "org.gnome.ScreenSaver"

#define MAINWINDOW_DBUS_FD_PM_SERVICE	"org.freedesktop.PowerManagement"
#define MAINWINDOW_DBUS_FD_PM_PATH		"/org/freedesktop/PowerManagement/Inhibit"
#define MAINWINDOW_DBUS_FD_PM_INTERFACE	"org.freedesktop.PowerManagement.Inhibit"

#define MAINWINDOW_DBUS_GSESS_PM_SERVICE	"org.gnome.SessionManager"
#define MAINWINDOW_DBUS_GSESS_PM_PATH		"/org/gnome/SessionManager"
#define MAINWINDOW_DBUS_GSESS_PM_INTERFACE	"org.gnome.SessionManager"

class MainWindow
{
public:
	MainWindow(App *Application, std::string *GladeFilename, Config *Cfg);
	virtual ~MainWindow();

	Gtk::Window *GetGtkWindow() {return GtkWindow;}

private:
	Config *Cfg;

	void on_FileNewAircraft_activate();
	void on_FileNewRoute_activate();
	void on_FileNewWxData_activate();
	void on_FileOpenAircraft_activate();
	void on_FileOpenChart_activate();
	void on_FileOpenRoute_activate();
	void on_FileOpenWxData_activate();
	void on_FileClose_activate();
	void on_FileDataManager_activate();
	void on_FileConfigure_activate();
	void on_FileFlashlight_activate();
	void on_FilePrint_activate();
	void on_FileQuit_activate();
	void on_ViewZoomIn_activate();
	void on_ViewZoomOut_activate();
	void on_ViewZoom30Pct_activate();
	void on_ViewZoom60Pct_activate();
	void on_ViewZoom100Pct_activate();
	void on_ViewZoom150Pct_activate();
	void on_ViewZoom200Pct_activate();
	void on_ViewZoom400Pct_activate();
	void on_ViewPanLeft_activate();
	void on_ViewPanRight_activate();
	void on_ViewPanUp_activate();
	void on_ViewPanDown_activate();
	void on_ViewPanFastLeft_activate();
	void on_ViewPanFastRight_activate();
	void on_ViewPanFastUp_activate();
	void on_ViewPanFastDown_activate();
	void on_ViewGoto_activate();
	void on_ViewFullscreen_activate();

	void on_EditAircraft_activate();
	void on_EditRoute_activate();

	void on_SearchAirports_activate();
	void on_SearchApproaches_activate();

	void on_GPSActive_activate();
	void on_GPSStartgpsd_activate();

	void on_HelpDisclaimer_activate();
	void on_HelpLicense_activate();
	void on_HelpWarranty_activate();
	void on_HelpAbout_activate();

	void on_CancelButton_clicked();

	void SetRecentFilesMenu();
	void OpenGeoChart(std::string filename);
	bool GPSHandler(int timer_number);
	bool SetGPSActive(bool value=true);
	void HandleGpsdConnected();
	void HandleGpsdDisconnected();
	bool Startgpsd();
	void LinkMETARsToAirports(std::vector<Airport*> *Airports, METARs *METARData);

	bool DBusInit();
	bool DBusShutdown();
	bool InhibitSS();
	bool UnInhibitSS();
	bool InhibitPM();
	bool UnInhibitPM();

	Glib::RefPtr<Gnome::Glade::Xml> refXml;
	Gtk::Window *GtkWindow;
	Gtk::MenuItem *FileNewAircraft;
	Gtk::MenuItem *FileNewRoute;
	Gtk::MenuItem *FileNewWxData;
	Gtk::MenuItem *FileOpenAircraft;
	Gtk::MenuItem *FileOpenChart;
	Gtk::MenuItem *FileOpenRoute;
	Gtk::MenuItem *FileOpenWxData;
	Gtk::MenuItem *FileClose;
	Gtk::MenuItem *FileDataManager;
	Gtk::MenuItem *FileConfigure;
	Gtk::MenuItem *FileFlashlight;
	Gtk::MenuItem *FilePrint;
	Gtk::MenuItem *FileQuit;
	Gtk::MenuItem *EditAircraft;
	Gtk::MenuItem *EditRoute;
	Gtk::MenuItem *ViewMenu;
	Gtk::MenuItem *ViewZoomIn;
	Gtk::MenuItem *ViewZoomOut;
	Gtk::MenuItem *ViewZoom30Pct;
	Gtk::MenuItem *ViewZoom60Pct;
	Gtk::MenuItem *ViewZoom100Pct;
	Gtk::MenuItem *ViewZoom150Pct;
	Gtk::MenuItem *ViewZoom200Pct;
	Gtk::MenuItem *ViewZoom400Pct;
	Gtk::MenuItem *ViewPanLeft;
	Gtk::MenuItem *ViewPanRight;
	Gtk::MenuItem *ViewPanUp;
	Gtk::MenuItem *ViewPanDown;
	Gtk::MenuItem *ViewPanFastLeft;
	Gtk::MenuItem *ViewPanFastRight;
	Gtk::MenuItem *ViewPanFastUp;
	Gtk::MenuItem *ViewPanFastDown;
	Gtk::MenuItem *ViewGoto;
	Gtk::CheckMenuItem *ViewFullscreen;

	Gtk::MenuItem *SearchAirports;
	Gtk::MenuItem *SearchApproaches;

	Gtk::MenuItem *GPSMenu;
	Gtk::CheckMenuItem *GPSActive;
	Gtk::MenuItem *GPSStartgpsd;
	Gtk::CheckMenuItem *GPSAutoSelectChart;

	Gtk::MenuItem *HelpDisclaimer;
	Gtk::MenuItem *HelpLicense;
	Gtk::MenuItem *HelpWarranty;
	Gtk::MenuItem *HelpAbout;
	Gtk::Statusbar *Statusbar;
	ChartDrawingArea *DrawingArea;
	App *Application;
	std::string *GladeFilename;
	GPS *Gps;
	sigc::connection GPSHandlerConnection;
	time_t LastSectionalChange;
	Route RouteData;
	Aircraft *AircraftData;
	DataStore *Data;
	Shapefile *ShapefileData;
	bool CancelFlag;

#ifdef HAVE_LIBDBUS_1
	DBusGProxy *DBusInhibitSSProxy,*DBusInhibitFDPMProxy,*DBusInhibitGSessPMProxy;
	guint32 *DBusInhibitSSCookie,*DBusInhibitFDPMCookie,*DBusInhibitGSessPMCookie;
#endif
};

#endif /*MAINWINDOW_H_*/
