/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#ifndef RUNWAY_H_
#define RUNWAY_H_

#include <string>
#include <vector>
#include "NavAid.h"

class Runway
{
public:
	Runway();

	/* Accessors */
	unsigned int GetLength();
	void SetLength(unsigned int Length);
	unsigned int GetWidth();
	void SetWidth(unsigned int Width);
	unsigned int GetDisplaced();
	void SetDisplaced(unsigned int Displaced);
	unsigned int GetElevation();
	void SetElevation(unsigned int Elevation);
	double GetMagBearing();
	void SetMagBearing(double MagBearing);
	double GetLon();
	void SetLon(double Lon);
	double GetLat();
	void SetLat(double Lat);
	double GetGradient();
	void SetGradient(double Gradient);
	std::string GetNotes();
	void SetNotes(std::string Notes);
	std::string GetID();
	void SetID(std::string ID);
	void ClearNavAids();
	void AddNavAid(NavAid *newNavAid);
	std::vector<NavAid*> GetNavAids();
	std::string GetAirportICAO();
	bool ParseNFD(std::string Line);
	bool ParseXPlane850(std::string *Line, Runway *OtherEnd, std::string ICAO, int AirportElevation);
	bool MakeBinary(struct Runway_Binary_struct *BinaryData);
	void SetFromBinary(struct Runway_Binary_struct *BinaryData);

private:
	unsigned int Length,Width,Displaced;	/* All in feett */
	int Elevation;			/* Feet */
	double MagBearing;		/* Magnetic bearing */
	double Lon,Lat;
	double Gradient;
	std::string Notes;
	std::string ID;
	std::string AirportICAO;
	std::vector<NavAid*> NavAids;
};

/* Increment the version when the structure changes */
#define RUNWAY_BINARY_STRUCT_VER 1
#ifdef WIN32
#pragma pack(1)
#endif
struct Runway_Binary_struct {
	uint16_t Length,Width,Displaced;
	int16_t Elevation;
	uint16_t MagBearingx10;
	int32_t Lonx1000000,Latx1000000;
	int16_t Gradientx10;
	char Notes[30];
	char ID[10];
}
#ifndef WIN32
        __attribute__ ((packed))
#endif
;
#ifdef WIN32
#pragma pack()
#endif

#endif /*RUNWAY_H_*/
